/*
 * (C) COPYRIGHT 2014 CRZ
 */

/* Includes */

#include "hw_config.h"

#define BMA150_I2C_ADDR 0x70

#define BMA150_RA_CHIP_ID           0x00
#define BMA150_RA_VERSION           0x01
#define BMA150_RA_X_AXIS_LSB        0x02
#define BMA150_RA_X_AXIS_MSB        0x03
#define BMA150_RA_Y_AXIS_LSB        0x04
#define BMA150_RA_Y_AXIS_MSB        0x05
#define BMA150_RA_Z_AXIS_LSB        0x06
#define BMA150_RA_Z_AXIS_MSB        0x07
#define BMA150_RA_TEMP_RD           0x08
#define BMA150_RA_SMB150_STATUS     0x09
#define BMA150_RA_SMB150_CTRL       0x0a
#define BMA150_RA_SMB150_CONF1      0x0b
#define BMA150_RA_LG_THRESHOLD      0x0c
#define BMA150_RA_LG_DURATION       0x0d
#define BMA150_RA_HG_THRESHOLD      0x0e
#define BMA150_RA_HG_DURATION       0x0f
#define BMA150_RA_MOTION_THRS       0x10
#define BMA150_RA_HYSTERESIS        0x11
#define BMA150_RA_CUSTOMER1         0x12
#define BMA150_RA_CUSTOMER2         0x13
#define BMA150_RA_RANGE_BWIDTH      0x14
#define BMA150_RA_SMB150_CONF2      0x15
#define BMA150_RA_OFFS_GAIN_X       0x16
#define BMA150_RA_OFFS_GAIN_Y       0x17
#define BMA150_RA_OFFS_GAIN_Z       0x18
#define BMA150_RA_OFFS_GAIN_T       0x19
#define BMA150_RA_OFFSET_X          0x1a
#define BMA150_RA_OFFSET_Y          0x1b
#define BMA150_RA_OFFSET_Z          0x1c
#define BMA150_RA_OFFSET_T          0x1d

#define BMA150_CHIP_ID_BIT             2
#define BMA150_CHIP_ID_LENGTH          3

#define BMA150_X_AXIS_LSB_BIT          7
#define BMA150_X_AXIS_LSB_LENGTH       2
#define BMA150_X_NEW_DATA_BIT          0

#define BMA150_Y_AXIS_LSB_BIT          7
#define BMA150_Y_AXIS_LSB_LENGTH       2
#define BMA150_Y_NEW_DATA_BIT          0

#define BMA150_Z_AXIS_LSB_BIT          7
#define BMA150_Z_AXIS_LSB_LENGTH       2
#define BMA150_Z_NEW_DATA_BIT          0

#define BMA150_STATUS_HG_BIT           0
#define BMA150_STATUS_LG_BIT           1
#define BMA150_HG_LATCHED_BIT          2
#define BMA150_LG_LATCHED_BIT          3
#define BMA150_ALERT_PHASE_BIT         4
#define BMA150_ST_RESULT_BIT           7

#define BMA150_SLEEP_BIT               0
#define BMA150_SOFT_RESET_BIT          1
#define BMA150_ST0_BIT                 2
#define BMA150_ST1_BIT                 3
#define BMA150_EEW_BIT                 4
#define BMA150_UPDATE_IMAGE_BIT        5
#define BMA150_RESET_INT_BIT           6

#define BMA150_ENABLE_LG_BIT           0
#define BMA150_ENABLE_HG_BIT           1
#define BMA150_COUNTER_LG_BIT          3
#define BMA150_COUNTER_LG_LENGTH       2
#define BMA150_COUNTER_HG_BIT          5
#define BMA150_COUNTER_HG_LENGTH       2
#define BMA150_ANY_MOTION_BIT          6
#define BMA150_ALERT_BIT               7

#define BMA150_LG_HYST_BIT             2
#define BMA150_LG_HYST_LENGTH          3
#define BMA150_HG_HYST_BIT             5
#define BMA150_HG_HYST_LENGTH          3
#define BMA150_ANY_MOTION_DUR_BIT      7
#define BMA150_ANY_MOTION_DUR_LENGTH   2

#define BMA150_BANDWIDTH_BIT           2
#define BMA150_BANDWIDTH_LENGTH        3
#define BMA150_RANGE_BIT               4
#define BMA150_RANGE_LENGTH            2

#define BMA150_WAKE_UP_BIT             0
#define BMA150_WAKE_UP_PAUSE_BIT       2
#define BMA150_WAKE_UP_PAUSE_LENGTH    2
#define BMA150_SHADOW_DIS_BIT          3
#define BMA150_LATCH_INT_BIT           4
#define BMA150_NEW_DATA_INT_BIT        5
#define BMA150_ENABLE_ADV_INT_BIT      6
#define BMA150_SPI4_BIT                7

/* range and bandwidth */
#define BMA150_RANGE_2G                0
#define BMA150_RANGE_4G                1
#define BMA150_RANGE_8G                2

#define BMA150_BW_25HZ                 0
#define BMA150_BW_50HZ                 1
#define BMA150_BW_100HZ                2
#define BMA150_BW_190HZ                3
#define BMA150_BW_375HZ                4
#define BMA150_BW_750HZ                5
#define BMA150_BW_1500HZ               6

static uint8_t Read_1_Byte(uint8_t address)
{
    uint8_t Buffer_Tx;
    uint8_t Buffer_Rx;

    Buffer_Tx = address;

    I2C_Master_BufferWrite(I2C1, &Buffer_Tx, 1, Polling, BMA150_I2C_ADDR);
    I2C_Master_BufferRead(I2C1, &Buffer_Rx, 1, Polling, BMA150_I2C_ADDR);

    return Buffer_Rx;
}

static uint8_t Get_ChipID(void)
{
    return Read_1_Byte(BMA150_RA_CHIP_ID);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
void Read_3_Axis_Data(int16_t * xData_p, int16_t * yData_p, int16_t * zData_p)
{
    uint8_t Buffer_Tx;
    uint8_t Buffer_Rx[6];

    Buffer_Tx = BMA150_RA_X_AXIS_LSB;

    I2C_Master_BufferWrite(I2C1, &Buffer_Tx, 1, Polling, BMA150_I2C_ADDR);
    I2C_Master_BufferRead(I2C1, Buffer_Rx, 6, Polling, BMA150_I2C_ADDR);

    * xData_p = ((((int16_t)(int8_t)Buffer_Rx[1]) << 8) | Buffer_Rx[0]) >> 6;
    * yData_p = ((((int16_t)(int8_t)Buffer_Rx[3]) << 8) | Buffer_Rx[2]) >> 6;
    * zData_p = ((((int16_t)(int8_t)Buffer_Rx[5]) << 8) | Buffer_Rx[4]) >> 6;
}

void Test_3AXIS_BMA150(void)
{
    uint8_t  Buffer_Tx[2];
    int16_t  xData, yData, zData;

    printf("Get_ChipID: %X\n", Get_ChipID());

    Buffer_Tx[0] = BMA150_RA_RANGE_BWIDTH;
    Buffer_Tx[1] = (BMA150_RANGE_8G << 3) | BMA150_BW_1500HZ;
    I2C_Master_BufferWrite(I2C1, Buffer_Tx, 2, Polling, BMA150_I2C_ADDR);
    printf("Set Param BMA150_RANGE_8G, BMA150_BW_1500HZ\n");

    while(1)
    {
        Read_3_Axis_Data(&xData, &yData, &zData);
        printf("x: %d, y: %d, z: %d\n", xData, yData, zData);
        Delay(300);
    }
}

