/*
 * (C) COPYRIGHT 2014 CRZ
 */

/* Includes */

#include "hw_config.h"

/*****************************************/
/* Device Enumeration                    */					
/*****************************************/
#define GYRO_DEVICE_ID		0xD6
#define INTERFACE_I2C	0

/*****************************************/
/* I2C setup                             */					
/*****************************************/
#define SA0_CONNECTED_TO_VCC   (0x02)
#define SA0_CONNECTED_TO_GND   (0)
#define SA0_CONFIG             (SA0_CONNECTED_TO_VCC)
#define GYRO_SLAVE_ADDRESS          (GYRO_DEVICE_ID | SA0_CONFIG)

/*****************************************/
/* Register Definition							     */					
/*****************************************/
#define WHO_AM_I		0x0F
#define CTRL_REG1		0x20
#define CTRL_REG2		0x21
#define CTRL_REG3		0x22
#define CTRL_REG4		0x23
#define CTRL_REG5		0x24
#define HP_FILTER_RESET	0x25
#define STATUS_REG		0x27
#define OUT_X_L			0x28
#define OUT_X_H			0x29
#define OUT_Y_L			0x2A
#define OUT_Y_H			0x2B
#define OUT_Z_L			0x2C
#define OUT_Z_H			0x2D
#define INT1_CFG		0x30
#define INT1_SOURCE		0x31
#define INT1_THS		0x32
#define INT1_DURATION	0x33
#define INT2_CFG		0x34
#define INT2_SOURCE		0x35
#define INT2_THS		0x36
#define INT2_DURATION	0x37

typedef enum adc_data
{
	X_VALUE=0,
	Y_VALUE,
	Z_VALUE,

	NUM_OF_VALUE
} ADC_DATA;

static uint8_t Read_1_Byte(uint8_t address)
{
    uint8_t Buffer_Tx;
    uint8_t Buffer_Rx;

    Buffer_Tx = address;

    I2C_Master_BufferWrite(I2C1, &Buffer_Tx, 1, Polling, GYRO_SLAVE_ADDRESS);
    I2C_Master_BufferRead(I2C1, &Buffer_Rx, 1, Polling, GYRO_SLAVE_ADDRESS);

    return Buffer_Rx;
}

static uint8_t Get_ChipID(void)
{
    return Read_1_Byte(WHO_AM_I);
}

static void Write_1_Byte(uint8_t address, uint8_t data)
{
    uint8_t Buffer_Tx[2];

    Buffer_Tx[0] = address;
    Buffer_Tx[1] = data;

    I2C_Master_BufferWrite(I2C1, Buffer_Tx, 2, Polling, GYRO_SLAVE_ADDRESS);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
void GYRO_L3G4200_vInit (void)
{
    // Enable x, y, z and turn off power down:
    Write_1_Byte(CTRL_REG1, 0x0F);

    // If you'd like to adjust/use the HPF, you can edit the line below to configure CTRL_REG2:
    Write_1_Byte(CTRL_REG2, 0x0);

    // Configure CTRL_REG3 to generate data ready interrupt on INT2
    // No interrupts used on INT1, if you'd like to configure INT1
    // or INT2 otherwise, consult the datasheet:
    Write_1_Byte(CTRL_REG3, 0x08);

    // CTRL_REG5 controls high-pass filtering of outputs, use it
    // if you'd like:
    Write_1_Byte(CTRL_REG5, 0x0);
}

/*
* Description : none.
* Argument(s) : none.
* Return(s)   : none.
* Note(s)     : none.
*/
void Test_Gyro_L3G4200DH(void)
{
    u8 u8X_H, u8X_L, u8Y_H, u8Y_L, u8Z_H, u8Z_L;
    signed short n16Value[NUM_OF_VALUE]={0};

    GYRO_L3G4200_vInit();

    printf("Get_ChipID: 0x%X\n", Get_ChipID());

    while(1)
    {
        // ROLL
        u8X_H = Read_1_Byte(OUT_X_H);
        u8X_L = Read_1_Byte(OUT_X_L);

        // PITCH	
        u8Y_H = Read_1_Byte(OUT_Y_H);
        u8Y_L = Read_1_Byte(OUT_Y_L);

        // YAW
        u8Z_H = Read_1_Byte(OUT_Z_H);
        u8Z_L = Read_1_Byte(OUT_Z_L);

        n16Value[X_VALUE] = (signed short)((u8X_H << 8) | u8X_L);
        n16Value[Y_VALUE] = (signed short)((u8Y_H << 8) | u8Y_L);
        n16Value[Z_VALUE] = (signed short)((u8Z_H << 8) | u8Z_L);

        Delay(200);
        printf ("X: %d, Y: %d, Z: %d\n", n16Value[X_VALUE], n16Value[Y_VALUE], n16Value[Z_VALUE]);
    }
}

