/*
 * (C) COPYRIGHT 2009 CRZ
 */

/* Includes */

#include "hw_config.h"

/*
 * Defines
 */

/** defines the return parameter type of the BMP085_WR_FUNCTION
*/
#define BMP085_BUS_WR_RETURN_TYPE int8_t

/** defines the calling parameter types of the BMP085_WR_FUNCTION
*/
#define BMP085_BUS_WR_PARAM_TYPES uint8_t,uint8_t,uint8_t *,uint8_t


/** defines the return parameter type of the BMP085_WR_FUNCTION
*/
#define BMP085_BUS_RD_RETURN_TYPE int8_t

/** defines the calling parameter types of the BMP085_WR_FUNCTION
*/
#define BMP085_BUS_RD_PARAM_TYPES uint8_t,uint8_t,uint8_t *,uint8_t

/* register write and read delays */
#define BMP085_MDELAY_DATA_TYPE	uint32_t
#define BMP085_MDELAY_RETURN_TYPE  void

/*
	CHIP_TYPE CONSTANTS
*/
#define BMP085_CHIP_ID			0x55
#define BOSCH_PRESSURE_SMD500	05
#define BOSCH_PRESSURE_BMP085	85

/*
	BMP085 I2C Address
*/
#define BMP085_I2C_ADDR		(0xEE>>1)

/*
	SMB380 API error codes
*/
#define E_BMP_NULL_PTR		    (char)-127
#define E_BMP_COMM_RES		    (char)-1
#define E_BMP_OUT_OF_RANGE		(char)-2
#define E_SENSOR_NOT_DETECTED   (char) 0

/* 
 *	register definitions 	
 */
#define BMP085_PROM_START__ADDR		0xaa
#define BMP085_PROM_DATA__LEN		  22

#define BMP085_CHIP_ID_REG			0xD0
#define BMP085_VERSION_REG			0xD1

#define BMP085_CTRL_MEAS_REG		0xF4
#define BMP085_ADC_OUT_MSB_REG		0xF6
#define BMP085_ADC_OUT_LSB_REG		0xF7

#define BMP085_SOFT_RESET_REG		0xE0

#define BMP085_T_MEASURE        0x2E				// temperature measurent 
#define BMP085_P_MEASURE        0x34				// pressure measurement

#define BMP085_TEMP_CONVERSION_TIME 5			// TO be spec'd by GL or SB

/* SMD500 specific constants */
#define SMD500_PROM_START__ADDR		0xf8
#define SMD500_PROM_DATA__LEN		8

#define SMD500_PARAM_M1     -2218        //calibration parameter
#define SMD500_PARAM_M2      -457        //calibration parameter
#define SMD500_PARAM_M3     -1984        //calibration parameter
#define SMD500_PARAM_M4      8808        //calibration parameter
#define SMD500_PARAM_M5       496        //calibration parameter
#define SMD500_PARAM_M6      1415        //calibration parameter

#define SMD500_PARAM_MB     -4955        //calibration parameter
#define SMD500_PARAM_MC     11611        //calibration parameter
#define SMD500_PARAM_MD    -12166        //calibration parameter
#define SMD500_PARAM_ME    -17268        //calibration parameter
#define SMD500_PARAM_MF     -8970        //calibration parameter

#define SMD500_PARAM_MG      3038        //calibration parameter
#define SMD500_PARAM_MH     -7357        //calibration parameter
#define SMD500_PARAM_MI      3791        //calibration parameter
#define SMD500_PARAM_MJ     64385        //calibration parameter

#define SMD500_STANDBY          0   				// set the device in stand-by modus to reduce power consumption
#define SMD500_MASTERCLOCK_32768HZ    0x04		   		// external Master clock 32.768kHz
#define SMD500_MASTERCLOCK_1MHZ       0 				// external Master clock 1MHz
#define SMD500_T_RESOLUTION_13BIT     0				// 13 Bit resolution temperature
#define SMD500_T_RESOLUTION_16BIT     0x80				// 16 Bit resolution temperature
#define SMD500_T_MEASURE              0x6A				// temperature measurent 
#define SMD500_P_MEASURE              0xF0				// pressure measurement

#define SMD500_TEMP_CONVERSION_TIME_13    9
#define SMD500_TEMP_CONVERSION_TIME_16	  34

/* 
 *	bit slice positions in registers
 */

#define BMP085_CHIP_ID__POS		0
#define BMP085_CHIP_ID__MSK		0xFF
#define BMP085_CHIP_ID__LEN		8
#define BMP085_CHIP_ID__REG		BMP085_CHIP_ID_REG

#define BMP085_ML_VERSION__POS		0
#define BMP085_ML_VERSION__LEN		4
#define BMP085_ML_VERSION__MSK		0x0F
#define BMP085_ML_VERSION__REG		BMP085_VERSION_REG

#define BMP085_AL_VERSION__POS  	4
#define BMP085_AL_VERSION__LEN  	4
#define BMP085_AL_VERSION__MSK		0xF0
#define BMP085_AL_VERSION__REG		BMP085_VERSION_REG

/* LG/HG thresholds are in LSB and depend on RANGE setting */
/* no range check on threshold calculation */

#define BMP085_GET_BITSLICE(regvar, bitname)\
			(regvar & bitname##__MSK) >> bitname##__POS

#define BMP085_SET_BITSLICE(regvar, bitname, val)\
		  (regvar & ~bitname##__MSK) | ((val<<bitname##__POS)&bitname##__MSK)  

/*
 * Data Types
 */

/** this structure holds all device specific calibration parameters 
*/
typedef struct {
   int16_t ac1;
   int16_t ac2;
   int16_t ac3;
   uint16_t ac4;
   uint16_t ac5;
   uint16_t ac6;
   int16_t b1;
   int16_t b2;
   int16_t mb;
   int16_t mc;
   int16_t md;      		   
} bmp085_smd500_calibration_param_t;

/** BMP085 image registers data structure 
*/
typedef struct  {	
	bmp085_smd500_calibration_param_t cal_param;	
	uint8_t mode;
	uint8_t chip_id,	ml_version,	al_version;
	uint8_t dev_addr;	
	uint8_t sensortype;

	int64_t param_b5;
	int32_t number_of_samples;
	int16_t oversampling_setting;
	int16_t smd500_t_resolution, smd500_masterclock;
	BMP085_BUS_WR_RETURN_TYPE (*bus_write)( BMP085_BUS_WR_PARAM_TYPES );
	BMP085_BUS_RD_RETURN_TYPE (*bus_read)( BMP085_BUS_RD_PARAM_TYPES );
	BMP085_MDELAY_RETURN_TYPE (*delay_msec)( BMP085_MDELAY_DATA_TYPE );
} bmp085_t;

/*
 * Local Variables
 */
static bmp085_t bmp085;

static uint8_t Read_1_Byte(uint8_t address)
{
    uint8_t Buffer_Tx;
    uint8_t Buffer_Rx;

    Buffer_Tx = address;

    I2C_Master_BufferWrite(I2C1, &Buffer_Tx, 1, Polling, bmp085.dev_addr);
    I2C_Master_BufferRead(I2C1, &Buffer_Rx, 1, Polling, bmp085.dev_addr);

    return Buffer_Rx;
}

static Status Read_N_Bytes(uint8_t address, uint8_t * Buffer_Rx_p, uint32_t NumByteToRead)
{
    uint8_t Buffer_Tx;

    Buffer_Tx = address;

    I2C_Master_BufferWrite(I2C1, &Buffer_Tx, 1, Polling, bmp085.dev_addr);
    return I2C_Master_BufferRead(I2C1, Buffer_Rx_p, NumByteToRead, Polling, bmp085.dev_addr);
}

static uint8_t Get_ChipID(void)
{
    return Read_1_Byte(BMP085_CHIP_ID__REG);
}

static void Write_1_Byte(uint8_t address, uint8_t data)
{
    uint8_t Buffer_Tx[2];

    Buffer_Tx[0] = address;
    Buffer_Tx[1] = data;

    I2C_Master_BufferWrite(I2C1, Buffer_Tx, 2, Polling, bmp085.dev_addr);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
int64_t bmp085_get_pressure(uint64_t up)
{
    long pressure,x1,x2,x3,b3,b6;
    unsigned long b4, b7;

    b6 = bmp085.param_b5 - 4000;

    //*****calculate B3************
    x1 = (b6*b6) >> 12;	 	 
    x1 *= bmp085.cal_param.b2;
    x1 >>=11;
    x2 = (bmp085.cal_param.ac2*b6);
    x2 >>=11;
    x3 = x1 +x2;
    b3 = (((((long)bmp085.cal_param.ac1 )*4 + x3) <<bmp085.oversampling_setting) + 2) >> 2;

    //*****calculate B4************
    x1 = (bmp085.cal_param.ac3* b6) >> 13;
    x2 = (bmp085.cal_param.b1 * ((b6*b6) >> 12) ) >> 16;
    x3 = ((x1 + x2) + 2) >> 2;
    b4 = (bmp085.cal_param.ac4 * (unsigned long) (x3 + 32768)) >> 15;

    b7 = ((unsigned long)(up - b3) * (50000>>bmp085.oversampling_setting));   
    if (b7 < 0x80000000)
    {
        pressure = (b7 << 1) / b4;
    }
    else
    { 
        pressure = (b7 / b4) << 1;
    }

    x1 = pressure >> 8;
    x1 *= x1;
    x1 = (x1 * SMD500_PARAM_MG) >> 16;
    x2 = (pressure * SMD500_PARAM_MH) >> 16;
    pressure += (x1 + x2 + SMD500_PARAM_MI) >> 4;	// pressure in Pa  

    return (pressure);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
int16_t bmp085_get_temperature(uint64_t ut) 
{
    short temperature;
    long x1,x2,x3,x4,y2,y3,y4;

    if(BOSCH_PRESSURE_BMP085 == bmp085.sensortype)
    {
        x1 = (((long) ut - (long) bmp085.cal_param.ac6) * (long) bmp085.cal_param.ac5) >> 15;
        x2 = ((long) bmp085.cal_param.mc << 11) / (x1 + bmp085.cal_param.md);
        bmp085.param_b5 = x1 + x2;
    }
    else // SMD500
    {
        if(SMD500_T_RESOLUTION_16BIT == bmp085.smd500_t_resolution) // check for SMD500 temp resolution mode
            x1 = ((unsigned long) ((( ut * SMD500_PARAM_MJ) >> 16) - bmp085.cal_param.ac6));
        else // 13BIT
            x1 = ut * 8  - bmp085.cal_param.ac6;

        x2 = (x1 * x1) >> 13;
        y2 = (SMD500_PARAM_MB * x2) >> 13;
        x3 = (x2 * x1) >> 16;
        y3 = (SMD500_PARAM_MC * x3) >> 12;
        x4 = (x2 * x2) >> 16;
        y4 = (SMD500_PARAM_MD * x4) >> 14;

        bmp085.param_b5 = (((bmp085.cal_param.ac5) * ( (2*x1) + y2 + y3 + y4))) >> 13; // temperature in (1/160)
    }
    temperature = ((bmp085.param_b5 + 8) >> 4);  // temperature in 0.1

    return (temperature);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
uint64_t bmp085_get_up(void)
{
    int32_t i;
    uint64_t up=0;
    uint8_t data[3];    
    uint8_t ctrl_reg_data;

    if(BMP085_CHIP_ID == bmp085.chip_id)
    { 
        ctrl_reg_data = BMP085_P_MEASURE + (bmp085.oversampling_setting << 6);

        Write_1_Byte(BMP085_CTRL_MEAS_REG, ctrl_reg_data);
        Delay(2 + (3 << bmp085.oversampling_setting));
        Read_N_Bytes(BMP085_ADC_OUT_MSB_REG, data, 3);
        up = (((unsigned long) data[0] << 16) | ((unsigned long) data[1] << 8) | (unsigned long) data[2])
            >> (8-bmp085.oversampling_setting);
        bmp085.number_of_samples = 1;
    }
    else 
    {
        // SMD500 
        ctrl_reg_data = SMD500_P_MEASURE + bmp085.smd500_masterclock;
        bmp085.number_of_samples = (1 << (bmp085.oversampling_setting));
        for(i = 0; i < bmp085.number_of_samples; i ++)
        {
            Write_1_Byte(BMP085_CTRL_MEAS_REG, ctrl_reg_data);
            Delay(34);
            Read_N_Bytes(BMP085_ADC_OUT_MSB_REG, data, 2);
            up += (((unsigned long)data[0] <<8) | (unsigned long)data[1]);
        }    
    }

    return (up);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
uint16_t bmp085_get_ut (void)
{
    uint16_t ut;
    uint8_t data[2];
    uint8_t ctrl_reg_data;
    int32_t wait_time;

    if (bmp085.chip_id == BMP085_CHIP_ID)            /* get bitslice */
    { 
        ctrl_reg_data = BMP085_T_MEASURE;
        wait_time = BMP085_TEMP_CONVERSION_TIME;
    } 
    else 
    {       
        ctrl_reg_data = SMD500_T_MEASURE + bmp085.smd500_t_resolution + bmp085.smd500_masterclock;
        // wait_time can be 9 ms for 13 bit smd500_t_resolution
        if(SMD500_T_RESOLUTION_13BIT == bmp085.smd500_t_resolution)
            wait_time = SMD500_TEMP_CONVERSION_TIME_13;
        else
            wait_time = SMD500_TEMP_CONVERSION_TIME_16;
    }
    Write_1_Byte(BMP085_CTRL_MEAS_REG, ctrl_reg_data);

    Delay(wait_time);
    Read_N_Bytes(BMP085_ADC_OUT_MSB_REG, data, 2);

    ut = (data[0] <<8) | data[1];

    return (ut);
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
void smd500_get_cal_param(void) 
{
    uint8_t data[SMD500_PROM_DATA__LEN];    

    Read_N_Bytes(SMD500_PROM_START__ADDR, data, SMD500_PROM_DATA__LEN);

    bmp085.cal_param.ac1 = (unsigned short) ((data[0] <<8) | data[1]) & 0x3FFF;
    bmp085.cal_param.ac2 = ((data[0] & 0xC0) >>6) + ((data[2]& ~0x01)<<1);
    bmp085.cal_param.ac3 = ((data[2] & 0x01)<<8) + data[3];
    bmp085.cal_param.ac4 = ((data[4] & 0x1f)<<8) + data[5];
    bmp085.cal_param.ac5 = ((data[4] & ~0x1f)>>5) + ((data[6] & 0xf0)>>1);
    bmp085.cal_param.ac6 = ((data[6] & 0x0f)<<8) + data[7];
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
void bmp085_get_cal_param(void)
{
    uint8_t data[BMP085_PROM_DATA__LEN];

    Read_N_Bytes(BMP085_PROM_START__ADDR, data, BMP085_PROM_DATA__LEN);

    /*parameters AC1-AC6*/
    bmp085.cal_param.ac1 =  (data[0] <<8) | data[1];
    bmp085.cal_param.ac2 =  (data[2] <<8) | data[3];
    bmp085.cal_param.ac3 =  (data[4] <<8) | data[5];
    bmp085.cal_param.ac4 =  (data[6] <<8) | data[7];
    bmp085.cal_param.ac5 =  (data[8] <<8) | data[9];
    bmp085.cal_param.ac6 = (data[10] <<8) | data[11];

    /*parameters B1,B2*/
    bmp085.cal_param.b1 =  (data[12] <<8) | data[13];
    bmp085.cal_param.b2 =  (data[14] <<8) | data[15];

    /*parameters MB,MC,MD*/
    bmp085.cal_param.mb =  (data[16] <<8) | data[17];
    bmp085.cal_param.mc =  (data[18] <<8) | data[19];
    bmp085.cal_param.md =  (data[20] <<8) | data[21];
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
void bmp085_init(void)
{
    uint8_t data;
    int64_t dummy;

    bmp085.sensortype = E_SENSOR_NOT_DETECTED;
    bmp085.dev_addr   = BMP085_I2C_ADDR << 1;  /* preset BMP085 I2C_addr */

    data = Get_ChipID();
    bmp085.chip_id = BMP085_GET_BITSLICE(data, BMP085_CHIP_ID);  
    printf("Get_ChipID: 0x%X\n", bmp085.chip_id);

    bmp085.number_of_samples = 1;  
    bmp085.oversampling_setting=0;

    if(bmp085.chip_id == BMP085_CHIP_ID)
    {
        bmp085.sensortype = BOSCH_PRESSURE_BMP085;

        data = Read_1_Byte(BMP085_VERSION_REG); /* read Version reg */

        bmp085.ml_version = BMP085_GET_BITSLICE(data, BMP085_ML_VERSION);        /* get ML Version */
        bmp085.al_version = BMP085_GET_BITSLICE(data, BMP085_AL_VERSION);        /* get AL Version */

        printf("ml_version: 0x%X, al_version: 0x%X\n", bmp085.ml_version, bmp085.al_version);

        bmp085_get_cal_param(); /* readout bmp085 calibparam structure */
    } 
    else 
    {
        bmp085.sensortype = BOSCH_PRESSURE_SMD500;
        bmp085.smd500_t_resolution = SMD500_T_RESOLUTION_16BIT;
        bmp085.smd500_masterclock = SMD500_MASTERCLOCK_32768HZ;

        smd500_get_cal_param();

        /* calculate B1*/
        dummy =  (bmp085.cal_param.ac3 + SMD500_PARAM_M3);
        dummy *= SMD500_PARAM_ME;
        dummy >>= 11;
        dummy += SMD500_PARAM_MF;   
        bmp085.cal_param.b1 = dummy;

        /* calculate B2*/
        dummy = (bmp085.cal_param.ac2 + SMD500_PARAM_M2);
        dummy *= bmp085.cal_param.b1;
        dummy >>= 4;
        dummy /= (bmp085.cal_param.ac3 + SMD500_PARAM_M3);   // calculate B2      
        bmp085.cal_param.b2 = dummy;

        bmp085.sensortype = BOSCH_PRESSURE_SMD500;

        bmp085.cal_param.ac1 += SMD500_PARAM_M1;
        bmp085.cal_param.ac1 <<= 1;

        bmp085.cal_param.ac2 += SMD500_PARAM_M2;
        bmp085.cal_param.ac2 <<= 2;

        bmp085.cal_param.ac3 += SMD500_PARAM_M3;
        bmp085.cal_param.ac3 <<= 3;

        bmp085.cal_param.ac4 += SMD500_PARAM_M4;
        bmp085.cal_param.ac4 <<= 1;

        bmp085.cal_param.ac5 += SMD500_PARAM_M5;
        bmp085.cal_param.ac5 <<= 3;

        bmp085.cal_param.ac6 += SMD500_PARAM_M6;
        bmp085.cal_param.ac6 <<= 3;

        bmp085.cal_param.mb = SMD500_PARAM_MB;
        bmp085.cal_param.mc = SMD500_PARAM_MC;
        bmp085.cal_param.md = SMD500_PARAM_MD;   
    }
}

/*
 * Description : none.
 * Argument(s) : none.
 * Return(s)   : none.
 * Note(s)     : none.
 */
void Test_Pressure_BMP085(void)
{
    uint64_t ut, up;
    int64_t dis_ut, dis_up;
//    int32_t dis_ut1, dis_up1;
    int32_t dis_ut2, dis_up2;

    bmp085_init();

    while(1)
    {
        ut = bmp085_get_ut();
        up = bmp085_get_up();

        // printf("ut: %lld, up: %lld\n", ut, up);
        
        dis_ut = bmp085_get_temperature(ut);
        dis_up = bmp085_get_pressure(up);

        // printf("dis_ut: %lld, dis_up: %lld\n", dis_ut, dis_up);
        
//        dis_ut1 = (int32_t)(dis_ut>>32);
        dis_ut2 = (int32_t)(dis_ut&0xffffffff);    

//        dis_up1 = (int32_t)(dis_up>>32);
        dis_up2 = (int32_t)(dis_up&0xffffffff);        

        printf("Temperature: %3.2f, Pressure: %4.2f hPa\n", (float)dis_ut2 / 10, (float)dis_up2 / 100);
        Delay(200);
    }
}

