/*
 * (C) COPYRIGHT 2012 CRZ
 *
 * File Name : hw_config.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 04/12/2012
 */

/* includes */

#include "hw_config.h"

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/

#define SYSTEMTICK_PERIOD_MS  10

/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/

__IO uint32_t LocalTime = 0; /* this variable is used to create a time reference incremented by 10ms */
uint32_t timingdelay;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Inserts a delay time.
  * @param  nCount: number of 10ms periods to wait for.
  * @retval None
  */
void Delay(uint32_t nCount)
{
    /* Capture the current local time */
    timingdelay = LocalTime + nCount;  

    /* wait until the desired delay finish */  
    while(timingdelay > LocalTime)
    {     
    }
}

/**
  * @brief  Updates the system local time
  * @param  None
  * @retval None
  */
void Time_Update(void)
{
    LocalTime += SYSTEMTICK_PERIOD_MS;
}

void System_Information()
{
    RCC_ClocksTypeDef  rcc_clocks;

    RCC_GetClocksFreq(&rcc_clocks);

    printf("SYSCLK_Frequency = %d\n",rcc_clocks.SYSCLK_Frequency );
    printf("HCLK_Frequency = %d\n",rcc_clocks.HCLK_Frequency );
    printf("PCLK1_Frequency = %d\n",rcc_clocks.PCLK1_Frequency );
    printf("PCLK2_Frequency = %d\n",rcc_clocks.PCLK2_Frequency );
}

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
    uint8_t ch;

    /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f2xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f2xx.c file
     */

    LED_Initialization();
    Key_Initialization();
    USART_Debug_Initialization();
    printf("USART_Debug_Initialization done.\n");
    printf("Mango-M32F2 Test - %s, %s\n", __DATE__, __TIME__);

    SysTick_Initialization();
    printf("SysTick_Initialization done.\n");

    /* Configure I2C */
    I2C_LowLevel_Init();
    printf("I2C_LowLevel_Init() done\n");

    while(1)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("0> System Information\n");
        printf("---------------------\n");
        printf("1> Test LED\n");
#ifdef ETHERNET_TEST
        printf("2> Test HTTP Server\n");
#endif
#ifdef USB_TEST
#ifdef USB_TEST_DEVICE_VCP
        printf("2> Test USB Device VCP\n");
#endif /* USB_TEST_DEVICE_VCP */
#ifdef USB_TEST_HOST_MSC
        printf("2> Test USB Host MSC\n");
#endif /* USB_TEST_HOST_MSC */
#ifdef USB_TEST_HOST_HID
        printf("2> Test USB Host HID\n");
#endif /* USB_TEST_HOST_HID */
#endif /* USB_TEST */
        printf("---------------------\n");
        printf("3> Test Light BH1600\n");
        printf("4> Test 3-AXIS BMA150\n");
        printf("5> Test Pressure BMP085\n");
        printf("6> Test Gyro L3G4200DH\n");
        printf("7> Test Remocon ROM-WT138\n");
        printf("8> Test GPS\n");
        printf("---------------------\n");
        printf("x> quit\n\n");

        ch = DebugUart_GetChar();
        printf(" is selected\n\n");

        switch((char)ch)
        {
        case '0':
            System_Information();
            break;

        case '1':
            Test_LED();
            break;

        case '2':
#ifdef ETHERNET_TEST
            Test_HttpServer();
#endif
#ifdef USB_TEST
#ifdef USB_TEST_DEVICE_VCP
            Test_USB_Device_VCP();
#endif /* USB_TEST_DEVICE_VCP */
#ifdef USB_TEST_HOST_MSC
            Test_USB_Host_MSC();
#endif /* USB_TEST_HOST_MSC */
#ifdef USB_TEST_HOST_HID
            Test_USB_Host_HID();
#endif /* USB_TEST_HOST_HID */
#endif /* USB_TEST */
            break;

        case '3':
            Test_Light_BH1600();
            break;

        case '4':
            Test_3AXIS_BMA150();
            break;

        case '5':
            Test_Pressure_BMP085();
            break;

        case '6':
            Test_Gyro_L3G4200DH();
            break;

        case '7':
            Test_Remocon_ROM_WT138();
            break;

        case '8':
            Test_GPS_Com();
            break;
        }

        if('x' == (char)ch)
        {
            break;
        }
    }
}

